
-- ----------------------------------------------------------
--   Useful information goes here at the top.
-- ----------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity target_game is
  port 
   (
    clk_in50       : in  std_logic; --50MHz
    pb_in          : in  std_logic; --low-active
    --
    led            : out std_logic;
    led_diag       : out std_logic_vector(5 downto 1)
   );
end entity target_game;

architecture Behavioral of target_game is

   component pll_x1 is
   port
   (
      inclk0   :  IN STD_LOGIC  := '0';
      c0       : OUT STD_LOGIC ;
      locked   : OUT STD_LOGIC 
    );
   end component;

   component debounce is
   port 
     (
       clk            : in  std_logic;
       pb_in          : in  std_logic;
       ms_tick        : in  std_logic;
       pb_debnced     : out std_logic
     );
   end component;

   component timer is
   port 
     (
       clk            : in  std_logic;
       timer_load     : in  std_logic_vector(15 downto 0); --units of milliseconds
       timer_start    : in  std_logic;
       ms_tick        : in  std_logic;
       timer_blink    : out std_logic;
       timer_done     : out std_logic
     );
   end component;

   constant MS_LOAD      : unsigned(15 downto 0) := X"C350"; --1ms at 50MHz
-- constant TAR_TIME     : std_logic_vector(15 downto 0) := X"0100"; --256
   constant TAR_TIME     : std_logic_vector(15 downto 0) := X"0400"; --1k, one second
   constant BLINK_TIME   : std_logic_vector(15 downto 0) := X"0800"; --2K (2 seconds)
   constant PAUSE_TIME   : std_logic_vector(15 downto 0) := X"1000"; --4K (4 seconds)

   signal clk50_pll      : std_logic;
   signal clk            : std_logic;
   signal ms_count       : unsigned(15 downto 0) := X"0001";
   signal ms_tick        : std_logic := '0';
   signal timer_load     : std_logic_vector(15 downto 0);
   signal span           : std_logic_vector(15 downto 0);
   signal timer_start    : std_logic;
   signal timer_done     : std_logic;
   signal pb_debnced     : std_logic;
   signal make_span      : std_logic;
   signal led_lcl        : std_logic;
   signal timer_blink    : std_logic;
   signal pseudo_sr      : std_logic_vector(1 to 10) := "1110100010";
   signal pseudo_seg     : std_logic_vector(3 downto 0);
   
   type state_type is ( create,
                        start,
                        led_span,
                        load_wait,
                        wait_span,
                        load_tar,
                        target,
                        load_blink,
                        blink,
                        load_pause,
                        pause
                      );

   signal state : state_type := create;
   
begin

   -- this cleans up the input clock
   pll_x1_ins : pll_x1
   port map
    (
      inclk0   => clk_in50,
      c0       => clk50_pll,
      locked   => open
    );

   clk <= clk50_pll;

    
   millisecond : process(clk) 
   begin
      if rising_edge(clk) then
         if (ms_count = X"0001") then
            ms_count <= MS_LOAD;
            ms_tick  <= '1';
         else
            ms_count <= ms_count - 1;
            ms_tick  <= '0';
         end if;
      end if;
   end process;

   state_machine : process(clk) 
   begin
      if rising_edge(clk) then
         case(state) is
            when create =>                                  state <= start;
            when start =>                                   state <= led_span;
            when led_span =>  if (timer_done = '1') then    state <= load_wait; end if;
            when load_wait =>                               state <= wait_span;
            when wait_span => if (timer_done = '1') then    state <= load_tar; end if;
            when load_tar =>                                state <= target;
            when target =>    if (pb_debnced = '1') then    state <= load_blink;
                              elsif (timer_done = '1') then state <= load_pause; 
                              end if;
            when load_blink =>                              state <= blink;
            when blink =>     if (timer_done = '1') then    state <= load_pause; end if;
            when load_pause =>                              state <= pause;
            when pause =>     if (timer_done = '1') then    state <= create; end if;
            when others => state <= create;
         end case;
      end if;
   end process;
   
   state_outputs : process(clk) 
   begin
      if rising_edge(clk) then
         make_span   <= '0';
         timer_start <= '0';
         led_lcl     <= '0';
         --
         case(state) is
            when create =>     make_span <= '1';
            when start =>      timer_load <= span;
                               timer_start <= '1';
            when led_span =>   led_lcl <= '1';
            when load_wait =>  timer_load <= span; 
                               timer_start <= '1';
            when load_tar =>   timer_load <= TAR_TIME;  
                               timer_start <= '1';
            when load_blink => timer_load <= BLINK_TIME;
                               timer_start <= '1';
            when blink =>      led_lcl <= timer_blink;
            when load_pause => timer_load <= PAUSE_TIME;
                               timer_start <= '1';
            when others => null;
         end case;
      end if;
   end process;
   
   led <= not led_lcl; --low-active LED

   pseudo_random : process(clk) 
   begin
      if rising_edge(clk) then
         if (make_span = '1') then
                         ------------ bit 1 ------------    -- bits 2-10 ----
            pseudo_sr <= (pseudo_sr(7) XOR pseudo_sr(10)) & pseudo_sr(1 to 9);
         end if;
      end if;
   end process;
   
   pseudo_seg <= pseudo_sr(1 to 4);
   
   span <= X"0200" when pseudo_seg = X"0" else
           "000" & pseudo_seg & '0' & X"00";  --span is 16 bits, up to 4 seconds = 4K ms = 12 bits
   
   debounce_ins : debounce
   port map
     (
       clk            => clk,         --in  std_logic;
       pb_in          => pb_in,       --in  std_logic;
       ms_tick        => ms_tick,     --in  std_logic;
       pb_debnced     => pb_debnced   --out std_logic
     );

   timer_inst : timer
   port map
     (
       clk            => clk,         --in  std_logic;
       timer_load     => timer_load,  --in  std_logic_vector(15 downto 0); --units of milliseconds
       timer_start    => timer_start, --in  std_logic;
       ms_tick        => ms_tick,     --in  std_logic;
       timer_blink    => timer_blink, --out std_logic;
       timer_done     => timer_done   --out std_logic
     );

--======================================
--    DIAGNOSTICS

-- led_diag(3 downto 1) <= 
--             "110" when state = create      else
--             "101" when state = led_span    else
--             "100" when state = wait_span   else
--             "011" when state = target      else
--             "010" when state = blink       else
--             "001";  -- state = pause

   led_diag(4 downto 1) <= not(pseudo_seg);
   led_diag(5) <= '1';
   
     
end architecture Behavioral;



