
-- ------------------------------------------------------
--   (Useful information.)
-- ------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity uart_rx is
  port (
        clk           : in   std_logic;
        rx_sig        : in   std_logic;
        uart_rx_dat   : out  std_logic_vector(7 downto 0);
        fifo_read     : in   std_logic;
        rrdy          : out  std_logic
        );
end entity uart_rx;

architecture Behavioral of uart_rx is

   -- baud constants assume a 50MHz clock
-- constant BAUD_9600 : std_logic_vector(15 downto 0) := X"1458";
   constant BAUD_9600 : std_logic_vector(15 downto 0) := X"0010";

   component baud is
     port (
            clk           : in   std_logic;
            baud_config   : in   std_logic_vector(15 downto 0);
            baud_en       : out  std_logic
          );
   end component;

   component fifo is
     port (
            clk           : in   std_logic;
            dat_in        : in   std_logic_vector(7 downto 0);
            f_write       : in   std_logic;
            dat_out       : out  std_logic_vector(7 downto 0);
            f_read        : in   std_logic;
            empty         : out  std_logic
          );
   end component;
   
   type rx_sm_type is ( rx_idle,
                        deglitch,
                        rx_start,
                        rx_bit_0,
                        rx_bit_1,
                        rx_bit_2,
                        rx_bit_3,
                        rx_bit_4,
                        rx_bit_5,
                        rx_bit_6,
                        rx_stop,
                        rx_done
                      );
   signal rx_sm : rx_sm_type;
   
   signal baud_div8           : std_logic_vector(15 downto 0);
   signal rx_osamp_en         : std_logic;
   signal sm_aux_cnt          : unsigned(3 downto 0) := X"0";
   signal bit_state           : std_logic;
   signal rx_sr               : std_logic_vector(6 downto 0) := "0000000";
   signal fifo_in             : std_logic_vector(7 downto 0) := X"00";
   signal fifo_write          : std_logic;
   signal fifo_empty          : std_logic;
   
begin

    --shift right x3 (by 8 divide)
   baud_div8 <= "000" & BAUD_9600(15 downto 3);

   baud_inst : baud
     port map
       (
         clk          => clk,        --in  std_logic;
         baud_config  => baud_div8,  --in  std_logic_vector(15 downto 0);
         baud_en      => rx_osamp_en --out std_logic
       );
   
   uart_rx_sm_aux_cnt : process(clk)
   begin
      if rising_edge(clk) then
         if (rx_osamp_en = '1') then
            if (    rx_sm = deglitch
                AND rx_sig = '0'
               ) then
               sm_aux_cnt <= X"3";
            elsif (sm_aux_cnt = X"1") then
               sm_aux_cnt <= X"8";
            elsif (rx_sm /= rx_idle) then
               sm_aux_cnt <= sm_aux_cnt - 1;
            end if;
         end if;
      end if;
   end process;
   
   uart_rx_sm : process(clk)
   begin
     if rising_edge(clk) then
        if (rx_osamp_en = '1') then
           case (rx_sm) is
             when rx_idle  => if (rx_sig = '0') then         rx_sm <= deglitch;   end if;
             when deglitch => if (rx_sig = '1') then         rx_sm <= rx_idle;
                              else                           rx_sm <= rx_start;   end if;                        
             when rx_start => if (rx_sig = '1') then         rx_sm <= rx_idle;
                              elsif (sm_aux_cnt = X"1") then rx_sm <= rx_bit_0;   end if;
             when rx_bit_0 => if (sm_aux_cnt = X"1") then    rx_sm <= rx_bit_1;   end if;
             when rx_bit_1 => if (sm_aux_cnt = X"1") then    rx_sm <= rx_bit_2;   end if;
             when rx_bit_2 => if (sm_aux_cnt = X"1") then    rx_sm <= rx_bit_3;   end if;
             when rx_bit_3 => if (sm_aux_cnt = X"1") then    rx_sm <= rx_bit_4;   end if;
             when rx_bit_4 => if (sm_aux_cnt = X"1") then    rx_sm <= rx_bit_5;   end if;
             when rx_bit_5 => if (sm_aux_cnt = X"1") then    rx_sm <= rx_bit_6;   end if;
             when rx_bit_6 => if (sm_aux_cnt = X"1") then    rx_sm <= rx_stop;    end if;
             when rx_stop  => if (sm_aux_cnt = X"1") then    rx_sm <= rx_done;    end if;
             when rx_done  =>                                rx_sm <= rx_idle;
             when others => null;
           end case;
        end if;
     end if;
   end process;

   bit_state <= '1' when (   rx_sm = rx_bit_0
                          OR rx_sm = rx_bit_1
                          OR rx_sm = rx_bit_2
                          OR rx_sm = rx_bit_3
                          OR rx_sm = rx_bit_4
                          OR rx_sm = rx_bit_5
                          OR rx_sm = rx_bit_6
                         )
                    else
                '0';
   
   shift_register : process(clk)
   begin
      if rising_edge(clk) then
         if (rx_osamp_en = '1') then
            if (    (bit_state = '1') 
                AND (sm_aux_cnt = X"1")
               ) then
               rx_sr <= rx_sig & rx_sr(6 downto 1);
            end if;
         end if;
      end if;
   end process;
   
   fifo_in    <= ('0' & rx_sr);
   fifo_write <= '1' when (rx_osamp_en = '1' AND rx_sm = rx_done)
                     else
                 '0';
   
   rx_fifo : fifo
     port map
     (
       clk     => clk,         --in   std_logic;
       dat_in  => fifo_in,     --in   std_logic_vector(7 downto 0);
       f_write => fifo_write,  --in   std_logic;
       dat_out => uart_rx_dat, --out  std_logic_vector(7 downto 0);
       f_read  => fifo_read,   --in   std_logic;
       empty   => fifo_empty   --out  std_logic
     );

   rrdy <= NOT fifo_empty;
   
end architecture Behavioral;


