
-- ------------------------------------------------------
--   (Useful information.)
-- ------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity shell is
  generic (BAUD_9600 : std_logic_vector(15 downto 0));
  port (
        clk           : in   std_logic;
        address       : in   std_logic_vector(11 downto 0);
        wr_en         : in   std_logic;
        data_wr       : in   std_logic_vector(7 downto 0);
        rd_en         : in   std_logic;
        data_rd       : out  std_logic_vector(7 downto 0);
        --
        uart_tx_sig   : out  std_logic;
        uart_rx_sig   : in   std_logic;
        leds          : out  std_logic_vector(3 downto 0)
        );
end entity shell;

architecture Behavioral of shell is

   component bus_intfc is 
     port (
            clk           : in   std_logic;
            address       : in   std_logic_vector(11 downto 0);
            wr_en         : in   std_logic;
            data_wr       : in   std_logic_vector(7 downto 0);
            rd_en         : in   std_logic;
            data_rd       : out  std_logic_vector(7 downto 0);
            --            
            trdy          : in   std_logic;
            uart_tx_go    : out  std_logic;
            uart_tx_dat   : out  std_logic_vector(7 downto 0);
            --            
            rrdy          : in   std_logic;
            fifo_read     : out  std_logic;
            uart_rx_dat   : in   std_logic_vector(7 downto 0);
            --            
            leds          : out  std_logic_vector(3 downto 0);
            --            
            memory_adr    : out  std_logic_vector(10 downto 0);
            memory_wr     : out  std_logic;
            memory_wr_dat : out  std_logic_vector(7 downto 0);
            memory_rd_dat : in   std_logic_vector(7 downto 0)
          );
   end component;

   component uart_tx is
     generic (BAUD_9600 : std_logic_vector(15 downto 0));
     port (
           clk           : in   std_logic;
           uart_tx_dat   : in   std_logic_vector(6 downto 0);
           uart_tx_go    : in   std_logic;
           tx_sig        : out  std_logic;
           trdy          : out  std_logic := '0'
          );
   end component;
   
   component uart_rx is
     generic (BAUD_9600 : std_logic_vector(15 downto 0));
     port (
            clk           : in   std_logic;
            rx_sig        : in   std_logic;
            uart_rx_dat   : out  std_logic_vector(7 downto 0);
            fifo_read     : in   std_logic;
            rrdy          : out  std_logic
          );
   end component;
   
   component memory is
     port (
            clk           : in   std_logic;
            memory_adr    : in   std_logic_vector(10 downto 0);
            memory_wr     : in   std_logic;
            memory_wr_dat : in   std_logic_vector(7 downto 0);
            memory_rd_dat : out  std_logic_vector(7 downto 0)
          );
   end component;

   signal uart_tx_go    : std_logic;
   signal uart_tx_dat_8 : std_logic_vector(7 downto 0);
   signal uart_rx_dat   : std_logic_vector(7 downto 0);
   signal trdy          : std_logic := '0';
   signal rrdy          : std_logic := '0';
   signal fifo_read     : std_logic;
   signal memory_adr    : std_logic_vector(10 downto 0);
   signal memory_wr     : std_logic;
   signal memory_wr_dat : std_logic_vector(7 downto 0);
   signal memory_rd_dat : std_logic_vector(7 downto 0);
   
begin

   bus_intfc_inst : bus_intfc
     port map 
          (
            clk           => clk,           --in   std_logic;
            address       => address,       --in   std_logic_vector(11 downto 0);
            wr_en         => wr_en,         --in   std_logic;
            data_wr       => data_wr,       --in   std_logic_vector(7 downto 0);
            rd_en         => rd_en,         --in   std_logic;
            data_rd       => data_rd,       --out  std_logic_vector(7 downto 0);
            --                              
            trdy          => trdy,          --in   std_logic;
            uart_tx_go    => uart_tx_go,    --out  std_logic;
            uart_tx_dat   => uart_tx_dat_8, --out  std_logic_vector(7 downto 0);
            --                              
            rrdy          => rrdy,          --in   std_logic;
            fifo_read     => fifo_read,     --out  std_logic;
            uart_rx_dat   => uart_rx_dat,   --in   std_logic_vector(7 downto 0);
            --                              
            leds          => leds,          --out  std_logic_vector(3 downto 0);
            --                              
            memory_adr    => memory_adr,    --out  std_logic_vector(10 downto 0);
            memory_wr     => memory_wr,     --out  std_logic;
            memory_wr_dat => memory_wr_dat, --out  std_logic_vector(7 downto 0);
            memory_rd_dat => memory_rd_dat  --in   std_logic_vector(7 downto 0)
          );

   uart_tx_inst : uart_tx
     generic map (BAUD_9600 => BAUD_9600)
     port map
          (
           clk           => clk,                       --in   std_logic;
           uart_tx_dat   => uart_tx_dat_8(6 downto 0), --in   std_logic_vector(6 downto 0);
           uart_tx_go    => uart_tx_go,                --in   std_logic;
           tx_sig        => uart_tx_sig,               --out  std_logic;
           trdy          => trdy                       --out  std_logic := '0'
          );
          
   uart_rx_inst : uart_rx
     generic map (BAUD_9600 => BAUD_9600)
     port map
          (
            clk           => clk,         --in   std_logic;
            rx_sig        => uart_rx_sig, --in   std_logic;
            uart_rx_dat   => uart_rx_dat, --out  std_logic_vector(7 downto 0);
            fifo_read     => fifo_read,   --in   std_logic;
            rrdy          => rrdy         --out  std_logic
          );
   
   memory_inst : memory
     port map
          (
            clk           => clk,           --in   std_logic;
            memory_adr    => memory_adr,    --in   std_logic_vector(10 downto 0);
            memory_wr     => memory_wr,     --in   std_logic;
            memory_wr_dat => memory_wr_dat, --in   std_logic_vector(7 downto 0);
            memory_rd_dat => memory_rd_dat  --out  std_logic_vector(7 downto 0)
          );
  
end architecture Behavioral;


